function gtLabels = classifyGroundTruth(nyuMatFile,outputFolder,viewScale,classId,overwrite)
%gtLabels = classifyGroundTruth('/n/fs/modelnet/NYUdataSet/dataset_detection3d.mat','/n/fs/modelnet/gt/toilet/',0.65,7,1)
mkdir(outputFolder);
    if ~exist('overwrite','var'), overwrite = 0; end
    
    if ~overwrite,
        fprintf('loading pre-computed data...\n');
        tmptmpoutputpathtmptmptmp = outputFolder;
        load([outputFolder '/all.mat']);
        outputFolder = tmptmpoutputpathtmptmptmp;
        load([outputFolder '/gtLabels.mat']);
        outputFolder = tmptmpoutputpathtmptmptmp;
        fprintf('data loaded.\n');
    else
        % load data file
        fprintf('loading data...\n');
        load(nyuMatFile);
        fprintf('data loaded.\n');

        % generate point cloud for each frame
        fprintf('generating point cloud...\n');
        points3d = cell(length(alignmentMatrices),1);
        parfor i = 1:length(alignmentMatrices),
            points3d{i} = rgb_plane2rgb_world(rawDepths(:,:,i));
            points3d{i} = get_aligned_point_cloud(points3d{i},alignmentMatrices{i});
        end
        R = cellfun(@(x) x([1 3 2],[1 3 2]),alignmentMatrices,'UniformOutput',false);

        % project ground truth bb to 2d
        fprintf('projecting bb into 2d...\n');
        nImages = length(R);
        width = size(rawDepths,2);
        height = size(rawDepths,1);
        imageBbIdx = cell(nImages,1);
        bbImageIdx = cell(nImages,1);
        for i = 1:nImages,
            imageBbIdx{i} = find([groundTruthBbs.imageNum] == i);
            bbImageIdx{i} = i * ones(length(imageBbIdx{i}),1);
        end
        gtBb2dDrawInCell = cell(length(groundTruthBbs),1);
        for i = 1:nImages,
            [~,gtBb2dDrawInCell{i}] = projectStructBbsTo2d(groundTruthBbs(imageBbIdx{i}),R{i});
        end
        bbImageIdx = cat(1,bbImageIdx{:});
        gtBb2dDraw = cat(1,gtBb2dDrawInCell{:});

        % classify gt bb
        fprintf('classifying ground truth bb...\n');
        gtLabels = zeros(length(groundTruthBbs),1);
        for i = 1:size(gtBb2dDraw,1),
            fprintf('classifying bb %d/%d...\n',i,length(groundTruthBbs));
            X = gtBb2dDraw(i,1:2:16);
            Y = gtBb2dDraw(i,2:2:16);
            XOff = X - min(X(:)) + 1;
            YOff = Y - min(Y(:)) + 1;
            idx = convhull(X,Y);
            maskObj = poly2mask(XOff(idx),YOff(idx),ceil(max(YOff)),ceil(max(XOff)));
            mask = poly2mask(X(idx),Y(idx),height,width);
            inSightRatio = sum(mask(:)) / sum(maskObj(:));
            hasDepth = rawDepths(:,:,groundTruthBbs(i).imageNum)~=0;
            [isNotOccluded,isInBb,centerDepth,fillRatio] = getBbMask(groundTruthBbs(i),points3d{bbImageIdx(i)},R{bbImageIdx(i)});
            isNotOccluded = reshape(isNotOccluded,height,width);
            isInBb = reshape(isInBb,height,width);
            occludedRatio = sum(isNotOccluded(mask(:))) / max(1,sum(mask(:)&hasDepth(:)));
            if centerDepth >= 7 || inSightRatio < 0.5,
                gtLabels(i) = 4;  % out of view
            elseif occludedRatio < 0.5 || occludedRatio * inSightRatio < 0.33
                gtLabels(i) = 2;  % occluded
            elseif fillRatio < 0.2
                gtLabels(i) = 3;  % too many nan
            else
                gtLabels(i) = 1;  % good
            end
        end
    end
    
    % visualize ground truth
    if exist('outputFolder','var'),
        % create output path
        if ~exist(outputFolder,'dir'), mkdir(outputFolder); end
        delete([outputFolder '/*.jpg']);
        delete([outputFolder '/gt_viewer.html']);
        
        save([outputFolder '/gtLabels.mat'],'gtLabels','-v6');
        save([outputFolder '/all.mat'],'-v6');
        
        % pick gt of current class
        if exist('classId','var'),
            isSameClass = [groundTruthBbs.classId] == classId;
            visGroundTruthBbs = groundTruthBbs(isSameClass);
            visBbImageIdx = bbImageIdx(isSameClass);
            visGtBb2dDraw = gtBb2dDraw(isSameClass,:);
            visLabels = gtLabels(isSameClass);
        end
        
        % same images
        fprintf('saving images...\n');
        bbColor = [0 1 0; 1 0 0; 0 0 1; 1 1 0] * 255;
        imageIds = unique([visGroundTruthBbs.imageNum]);
        gtImageNames = cell(length(imageIds),1);
        imagesInCell = cell(length(imageIds),1);
        for i = 1:length(imageIds),
            tmpR = images(:,:,1,imageIds(i)); tmpR(rawDepths(:,:,imageIds(i))==0) = 255;
            tmpG = images(:,:,2,imageIds(i)); tmpG(rawDepths(:,:,imageIds(i))==0) = 255;
            tmpB = images(:,:,3,imageIds(i)); tmpB(rawDepths(:,:,imageIds(i))==0) = 255;
            imagesInCell{i} = cat(3,tmpR,tmpG,tmpB);
        end
        parfor i = 1:length(imageIds),
            fprintf('saving image %d/%d...\n',i,length(imageIds));
            maskedImg = imagesInCell{i};
            gtBb2dDrawCurr = visGtBb2dDraw(visBbImageIdx==imageIds(i),:);
            labelsCurr = visLabels(visBbImageIdx==imageIds(i));
            for j = 1:size(gtBb2dDrawCurr,1),
                maskedImg = drawBbOnImage(maskedImg,gtBb2dDrawCurr(j,:),bbColor(labelsCurr(j),:));
            end
            gtImageNames{i} = sprintf('gt-image%05d.jpg',imageIds(i));
            imwrite(maskedImg,sprintf('%s/%s',outputFolder,gtImageNames{i}));
        end
        
        % generate html
        fprintf('generating html file...\n');
        htmlFileName = [outputFolder '/gt_viewer.html'];
        htmlFile = fopen(htmlFileName,'w');
        if htmlFile < 0, error('cannot create file %s.html',htmlFileName); end
        
        fprintf(htmlFile,'<html><head><title>3D Detection Viewer</title></head><body>\n');
        fprintf(htmlFile,'<center><h2>Ground Truth Viewer<br>\n');
        
        nImagesPerRow = 3;
        fprintf(htmlFile,'<table style="overflow:hidden;" cellpadding="10">\n');
        fprintf(htmlFile,'<tr></tr>\n');
        nRows = ceil(length(gtImageNames) / nImagesPerRow);
        for j = 1:nRows,
            % show images and detections
            fprintf(htmlFile,'<tr>\n');
            for i = 1:nImagesPerRow,
                entryIdx = (j-1)*nImagesPerRow+i;
                if entryIdx > length(gtImageNames), break; end
                img = imread([outputFolder '/' gtImageNames{entryIdx}]);
                width = round(viewScale * size(img,2));
                height = round(viewScale * size(img,1));
                fprintf(htmlFile,'\t<td><div style="position:relative;"><img src="%s" width="%d" height="%d" /></div></td>\n',gtImageNames{entryIdx},width,height);
            end
            fprintf(htmlFile,'</tr>\n');

            % show names
            fprintf(htmlFile,'<tr>\n');
            for i = 1:nImagesPerRow,
                entryIdx = (j-1)*nImagesPerRow+i;
                if entryIdx > length(gtImageNames), break; end
                fprintf(htmlFile,'\t<td align="center">%d. Test_%05d</td>\n',entryIdx,imageIds(entryIdx));
            end
            fprintf(htmlFile,'</tr>\n');

            % insert an empty row for spacing
            fprintf(htmlFile,'<tr><td colspan="%d">&nbsp;</td></tr>\n',nImagesPerRow);
        end
        fprintf(htmlFile,'</table>\n');

        fprintf(htmlFile,'</body></html>\n');
        fclose(htmlFile);
        
        % generate image crops
        ConstsDetection;
        mkdir([outputFolder '/' consts.classNames{classId}]);
        allClassNames = {'all','good','occ','nan','out'};
        for i = 1:length(allClassNames),
            mkdir([outputFolder '/' consts.classNames{classId} '/' allClassNames{i}]); delete([outputFolder '/' consts.classNames{classId} '/' allClassNames{i} '/*']); 
        end
        
        for i = 1:length(visGroundTruthBbs),
            img = imagesInCell{imageIds == visGroundTruthBbs(i).imageNum};
            height = size(img,1);
            width = size(img,2);
            X = visGtBb2dDraw(i,1:2:16);
            Y = visGtBb2dDraw(i,2:2:16);
            idx = convhull(X,Y);
            mask = poly2mask(X(idx),Y(idx),height,width);
            imgR = img(:,:,1); imgR(~mask) = 255;
            imgG = img(:,:,2); imgG(~mask) = 255;
            imgB = img(:,:,3); imgB(~mask) = 255;
            img = cat(3,imgR,imgG,imgB);
            img = img(max(1,floor(min(Y))):min(height,ceil(max(Y))),max(1,floor(min(X))):min(width,ceil(max(X))),:);
            imwrite(img,[outputFolder '/' consts.classNames{classId} '/' allClassNames{1} '/' sprintf('%03d_Test%05d.png',i,visGroundTruthBbs(i).imageNum)]);
            imwrite(img,[outputFolder '/' consts.classNames{classId} '/' allClassNames{visLabels(i)+1} '/' sprintf('%03d_Test%05d.png',i,visGroundTruthBbs(i).imageNum)]);
        end
    end
end