nyuMatFile = '/n/fs/modelnet/NYUdataSet/dataset_detection3d_new.mat';
gtLabelsFile = '/n/fs/modelnet/Code/Output/WebResult/gt/gtLabels.mat';
classId = 4;

labelNames = {'none','good','occlusion','special','bad annotation','missing depth','out of sight'};
labelColors = [0 0 0; 0 1 0; 1 0 0; 0 0 1; 1 0 1; 0 1 1; 1 1 0];

fprintf('loading mat files...\n');
load(nyuMatFile);
load(gtLabelsFile);

fprintf('computing 3d point cloud for each frame...\n');
points3d = cell(length(alignmentMatrices),1);
parfor i = 1:length(alignmentMatrices),
    points3d{i} = rgb_plane2rgb_world(rawDepths(:,:,i));
    points3d{i} = get_aligned_point_cloud(points3d{i},alignmentMatrices{i});
end

R = cellfun(@(x) x([1 3 2],[1 3 2]),alignmentMatrices,'UniformOutput',false);
% transfor old labels
% gtLabels(gtLabels==3) = 5;
% gtLabels(gtLabels==4) = 6;

fprintf('start labeling...\n');
bbIdx = find([groundTruthBbs.classId] == classId);
i = 1;
while 1,
    if i < 1 || i > length(bbIdx), break; end
    
    bbId = bbIdx(i);
    imageId = groundTruthBbs(bbId).imageNum;
    
    [~,gtBb2dDraw] = projectStructBbsTo2d(groundTruthBbs(bbId),R{imageId});
    [~,isInBb,~,~] = getBbMask(groundTruthBbs(bbId),points3d{imageId},R{imageId});
    
    % image with gt bb
    maskRatio = 1/3;
    tmpR = images(:,:,1,imageId); tmpR(isInBb) = (1-maskRatio)*tmpR(isInBb); tmpR(rawDepths(:,:,imageId)==0) = 255;
    tmpG = images(:,:,2,imageId); tmpG(isInBb) = (1-maskRatio)*tmpG(isInBb) + maskRatio*255; tmpG(rawDepths(:,:,imageId)==0) = 255;
    tmpB = images(:,:,3,imageId); tmpB(isInBb) = (1-maskRatio)*tmpB(isInBb); tmpB(rawDepths(:,:,imageId)==0) = 255;
    maskedImg = im2double(cat(3,tmpR,tmpG,tmpB));
    maskedImg = drawBbOnImage(maskedImg,gtBb2dDraw,labelColors(gtLabels(bbId)+1,:));
    
    % add label selection
    gap = ones(size(maskedImg,1),50,3);
    selectionRegion = ones(size(maskedImg,1),100,3);
    maskedImg = [maskedImg gap selectionRegion gap];
    labelTextGap = round(size(maskedImg,1)/6);
    labelTextY = round(labelTextGap/2):labelTextGap:size(maskedImg,1);
    
    clf;
    imshow(maskedImg); hold on; title(num2str(i));
    for j = 1:length(labelNames)-1,
        text(size(images,2)+size(gap,2), labelTextY(j), labelNames{j+1}, 'color', labelColors(j+1,:));
    end
    while 1,
        [x,y,key] = ginput(1);
        if key == 28,
            i = i - 1;
            break;
        elseif key == 29,
            i = i + 1;
            break;
        else
            [~,gtLabels(bbId)] = min(abs(y-labelTextY));
            maskedImg = drawBbOnImage(maskedImg,gtBb2dDraw,labelColors(gtLabels(bbId)+1,:));
            clf; imshow(maskedImg); hold on; title(num2str(i));
            for j = 1:length(labelNames)-1,
                text(size(images,2)+size(gap,2), labelTextY(j), labelNames{j+1}, 'color', labelColors(j+1,:));
            end
        end
    end
end

save('gtLabels_manual.mat','gtLabels');

